/*
 * kaffeinethumbnailer.cpp
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include <unistd.h>
#include <sys/resource.h>

#include <QCoreApplication>
#include <QDir>
#include <QImage>

#include "kaffeinethumbnailer.h"



KaffeineThumb::KaffeineThumb()
{
	raw_visual_t vis;
	char *vo_driver = "auto";
	char *ao_driver = "none";

	xine = xine_new();
	xine_config_load( xine, QString( QDir::homePath()+"/.xine/config" ).toLatin1().data() );
	xine_init( xine );

	vis.user_data = this;
	vis.raw_output_cb = &KaffeineThumb::raw_output_cb;
	vis.raw_overlay_cb = &KaffeineThumb::raw_overlay_cb;
	vis.supported_formats = XINE_VORAW_RGB;

	voPort = xine_open_video_driver( xine, vo_driver, XINE_VISUAL_TYPE_RAW, (void *)&vis );
	aoPort = xine_open_audio_driver( xine , ao_driver, NULL );

	stream = xine_stream_new( xine, aoPort, voPort );

	takingShot = false;
	thumbWidth = 100;
	memset( inBuf, 0, sizeof(inBuf) );
}



KaffeineThumb::~KaffeineThumb()
{
	xine_close( stream );
	xine_dispose( stream );
	xine_close_audio_driver( xine, aoPort );
	xine_close_video_driver( xine, voPort );
	xine_exit( xine );
}



bool KaffeineThumb::readStdin()
{
	while (1) {
		if ( inFile.readLine( inBuf, 1500 )>0 )
			return true;
		usleep( 10000 );
	}
}



void KaffeineThumb::takeShots( int pos, int width )
{
	thumbWidth = width;
	inFile.open( stdin, QIODevice::ReadOnly );
	while (1) {
		loop:
		readStdin();
		srcFile = QString::fromLocal8Bit( inBuf );
		srcFile = srcFile.left(srcFile.length()-1);
		//fprintf( stderr, "captured filename:%s %d\n", srcFile.toLocal8Bit().data(), srcFile.length() );
		readStdin();
		dstFile = QString::fromLocal8Bit( inBuf );
		dstFile = dstFile.left(dstFile.length()-1);
		//fprintf( stderr, "dest filename:%s %d\n", dstFile.toLocal8Bit().data(), dstFile.length() );
		takingShot = true;
		if ( !xine_open( stream, srcFile.toLocal8Bit().data() ) || !xine_play( stream, pos, 0 ) ) {
			fprintf( stderr, "unable to open:%s\n", srcFile.toLocal8Bit().data() );
			takingShot = false;
			goto loop;
		}
		for ( int i=0; i<300; ++i ) {
			if ( !takingShot ) {
				xine_stop( stream );
				fprintf( stderr, "shot taken:%s\n", srcFile.toLocal8Bit().data() );
				goto loop;
			}
			else
				usleep( 10000 );
		}
		fprintf( stderr, "shot failed:%s\n", srcFile.toLocal8Bit().data() );
		xine_stop( stream );
	}
}



void KaffeineThumb::takeShot( QString src, QString dst, int pos, int width )
{
	dstFile = dst;
	srcFile = src;
	takingShot = true;
	thumbWidth = width;
	if ( !xine_open( stream, srcFile.toLocal8Bit() ) || !xine_play( stream, pos, 0 ) ) {
		fprintf( stderr, "unable to open:%s\n", srcFile.toLocal8Bit().data() );
		return;
	}
	for ( int i=0; i<300; ++i ) {
		if ( !takingShot ) {
			xine_stop( stream );
			fprintf( stderr, "shot taken:%s\n", srcFile.toLocal8Bit().data() );
			return;
		}
		else
			usleep( 10000 );
	}
	fprintf( stderr, "shot failed:%s\n", srcFile.toLocal8Bit().data() );
	xine_stop( stream );
}


#include <QFile>
#include <QTextStream>
void KaffeineThumb::raw_output_cb( void *user_data, int frame_format, int frame_width,
	int frame_height, double frame_aspect, void *data0, void*, void* )
{
	KaffeineThumb *xt = (KaffeineThumb*)user_data;

	if ( !xt->takingShot ) {
		return;
	}
	if ( !data0 || frame_format!=XINE_VORAW_RGB ) {
		xt->takingShot = false;
		return;
	}

	unsigned char *buf = (unsigned char*)data0;
	unsigned char *dat = new unsigned char[frame_width*frame_height*4];
	unsigned char *tmp = dat;
	for ( int i=0; i<frame_width*frame_height; ++i ) {
		tmp[2] = buf[0];
		tmp[1] = buf[1];
		tmp[0] = buf[2];
		tmp[3] = 255;
		buf+=3;
		tmp+=4;
	}
	QImage img( dat, frame_width, frame_height, QImage::Format_RGB32 );
	qreal sx = (qreal)xt->thumbWidth/(qreal)frame_width;
	qreal sy = (qreal)xt->thumbWidth/(frame_aspect*(qreal)frame_height);
	if ( !img.transformed( QMatrix().scale( sx, sy ), Qt::SmoothTransformation ).save( xt->dstFile ) )
		fprintf(stderr, "save image failed=%s\n", xt->dstFile.toLocal8Bit().data());
	delete [] dat;

	/*
	for future xine-lib with XINE_VORAW_RGB32
	QImage img( (unsigned char*)data0, frame_width, frame_height, QImage::Format_RGB32 );
	qreal sx = (qreal)xt->thumbWidth/(qreal)frame_width;
	qreal sy = (qreal)xt->thumbWidth/(frame_aspect*(qreal)frame_height);
	if ( !img.transformed( QMatrix().scale( sx, sy ), Qt::SmoothTransformation ).save( xt->dstFile ) )
		fprintf(stderr, "save image failed=%s\n", xt->dstFile.toLocal8Bit().data());*/

	xt->takingShot = false;
}



int main( int argc, char **argv )
{
	if ( argc!=4 && argc!=5 ) {
		fprintf( stderr, "usage : kaffeinethumbnailer (sourceFilename) (destFilename) (seekPos(0-65535)) (thumbWidth)\n" );
		fprintf( stderr, "ex : kaffeinethumbnailer /home/me/movie.avi /home/me/movie.jpg 10000 400\n\n" );
		fprintf( stderr, "or    : kaffeinethumbnailer batchMode (seekPos) (thumbWidth)\n" );
		fprintf( stderr, "ex : kaffeinethumbnailer batchMode 10000 400\n\n" );
		fprintf( stderr, "In batchMode, you have to feed the standard input as follow:\n");
		fprintf( stderr, "     sourceFilename-1\n");
		fprintf( stderr, "     destFilename-1\n");
		fprintf( stderr, "     sourceFilename-2\n");
		fprintf( stderr, "     destFilename-2\n");
		fprintf( stderr, "     ...\n");
		return 1;
	}

	//FIXME  linux only?
	setpriority(PRIO_PROCESS, 0, 10);

	QCoreApplication app( argc, argv );
	KaffeineThumb kt;
	if ( argc==4 )
		kt.takeShots( QString(argv[2]).toInt(), QString(argv[3]).toInt() );
	else
		kt.takeShot( QString::fromLocal8Bit(argv[1]), QString::fromLocal8Bit(argv[2]), QString(argv[3]).toInt(), QString(argv[4]).toInt() );
	return 0;
}
