/*
 * osdcontrolbar.h
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "osdcontrolbar.h"



OsdControlBar::OsdControlBar( OsdConfig *conf ) : OsdAnim( conf )
{
	scroll = 0;
	positionBar = new OsdVScrollBar( conf );
	connect( positionBar, SIGNAL(scrolled(int)), this, SLOT(barScrolled(int)) );
	positionBar->setBarType( OsdVScrollBar::Horizontal );
	positionBar->setRange( 3600 );
}



OsdControlBar::~OsdControlBar()
{
	for ( int i=0; i<buttonList.size(); ++i )
		delete buttonList.at(i);
}



void OsdControlBar::appendButton( OsdButton *btn )
{
	buttonList.append( btn );
	connect( btn, SIGNAL(wantRepaint()), this, SIGNAL(wantRepaint()) );
}



void OsdControlBar::mouseMove( QPoint &p )
{
	if ( isHidden() || isMoving() )
		return;

	if ( !inMe( p.x(), p.y() ) && !hasFocus )
		return;
	else if ( !inMe( p.x(), p.y() ) )
		hasFocus = false;
	else
		hasFocus = true;

	if ( positionBar->isScrolling() ) {
		positionBar->mouseMove( p );
		return;
	}

	for ( int i=0; i<buttonList.size(); ++i ) {
		buttonList.at(i)->mouseMove( p );
	}
}



void OsdControlBar::wheelEvent( QWheelEvent *e )
{
	if ( !hasFocus || !positionBar->area().contains( e->x(), e->y() ) )
		return;
	int off = -e->delta()/8;
	if ( off<0 && scroll==0 )
		return;
	scroll+=off;
	if ( scroll<0 )
		scroll = 0;
	wantRepaint();
}



void OsdControlBar::barScrolled( int off )
{
	scroll = off;
	emit wantRepaint();
}



bool OsdControlBar::mouseClicked( QMouseEvent *e )
{
	if ( !inMe( e->x(), e->y() ) )
		return false;
	if ( positionBar->mouseClicked( e ) )
		return true;
	for ( int i=0; i<buttonList.size(); ++i ) {
		if ( buttonList.at(i)->mouseClicked( e ) )
			return true;
	}
	return false;
}



void OsdControlBar::mouseReleased( QMouseEvent *e )
{
	positionBar->mouseReleased( e );
}



void OsdControlBar::paint()
{
	if ( isHidden() && !isMoving() )
		return;

	//setTransform( p );

	config->func->drawQuad( rect, config->backgroundColor() );

	for ( int i=0; i<buttonList.size(); ++i ) {
		if ( buttonList.at(i)->x()>(rect.x()+rect.width()) )
			break;
		buttonList.at(i)->paint();
	}

	positionBar->setPos( scroll );
	positionBar->paint();

	//resetTransform( p );
}



void OsdControlBar::resize( int w, int h )
{
	int tmp=0;
	QFontMetrics fm( config->normalFont() );
	int r = fm.boundingRect( QRect(), Qt::AlignCenter, "Peje" ).height();
	r = (int) (r*1.3);
	if ( r<SCROLLBARSIZE )
		r = SCROLLBARSIZE;
	rect.setY( h-(r+(2*MARGINSIZE)) );
	rect.setHeight( r+(2*MARGINSIZE) );
	rect.setWidth( w );
	for ( int i=0; i<buttonList.size(); ++i ) {
		buttonList.at(i)->setGeometry( rect.x()+MARGINSIZE+tmp, rect.y()+MARGINSIZE, r, r );
		tmp += r+MARGINSIZE;
	}
	tmp += MARGINSIZE+rect.x()+MARGINSIZE;
	positionBar->setGeometry( tmp, rect.y()+(rect.height()-SCROLLBARSIZE)/2, w-MARGINSIZE-tmp, SCROLLBARSIZE );
}
