/*
 * osdthumbitem.cpp
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "osdthumbitem.h"



OsdThumbItem::OsdThumbItem( OsdConfig *conf, const QString &t, const QString &thumbpath, QImage *nothumb ) : OsdObject( conf )
{
	fileName = t;
	thumbFileName = thumbpath;
	noThumb = nothumb;
	texture = fnameTexture = 0;
	hasThumb = hasDefThumb = hasText = false;
	glGenTextures( 1, &texture );
	glGenTextures( 1, &fnameTexture );
}



OsdThumbItem::~OsdThumbItem()
{
	glDeleteTextures( 1, &texture );
	glDeleteTextures( 1, &fnameTexture );
}



bool OsdThumbItem::updateThumb( const QString &s )
{
	if ( s==thumbFileName ) {
		hasThumb = false;
		return true;
	}
	return false;
}



bool OsdThumbItem::load()
{
	QImage thumb;
	if ( !thumb.load( thumbFileName ) )
		return false;

 	int texW, texH;
	config->func->textureFromData( texture, thumb.width(), thumb.height(), thumb.bits(), texW, texH );
	if ( ((float)thumb.width()/thumb.height())>(4./3.) ) {
		float w = thumb.width()-(thumb.height()*4./3.);
		u = (w/2.)/texW;
		u1 = ((float)thumb.width()-(w/2.))/(float)texW;
	}
	else {
		u = 0;
		u1 = (float)thumb.width()/(float)texW;
	}
	v = 0;
	v1 = (float)thumb.height()/(float)texH;

	hasThumb = true;
	return true;
}



bool OsdThumbItem::loadDef()
{
	int texW, texH;
	config->func->textureFromData( texture, noThumb->width(), noThumb->height(), noThumb->bits(), texW, texH );
	u = 0;
	u1 = (float)noThumb->width()/(float)texW;
	v = 0;
	v1 = (float)noThumb->height()/(float)texH;

	hasDefThumb = true;
	return true;
}



void OsdThumbItem::paint( QRectF r )
{
	if ( !hasThumb && !load() ) {
		if ( !hasDefThumb && !loadDef()) {
			config->func->drawRect( r );
			return;
		}
	}
	config->func->drawQuadTexture( texture, r, u, v, u1, v1 );
	config->func->drawRect( r );
}



void OsdThumbItem::paintText( QRectF r )
{
	if ( !hasText ) {
		QFontMetrics fm( config->normalFont() );
		QRect fr = fm.boundingRect( QRect(), Qt::AlignVCenter, fileName );
		QImage img( fr.width(), fr.height(), QImage::Format_ARGB32 );
		QPainter p;
		p.begin( &img );
		memset( img.bits(), 0, img.width()*img.height()*4 );
		p.setFont( config->normalFont() );
		p.fillRect( 0, 0, img.width(), img.height(), config->listItemGrad0Color() );
		p.setPen( config->foregroundHighColor() );
		p.drawText( 0,0,img.width(),img.height(), Qt::AlignCenter, fileName );
		p.end();

		int texW, texH;
		config->func->textureFromData( fnameTexture, img.width(), img.height(), img.bits(), texW, texH );
		tu = 0;
		tu1 = (float)img.width()/(float)texW;
		tv = 0;
		tv1 = (float)img.height()/(float)texH;
		twidth = img.width();
		hasText = true;
	}
	if ( r.width()>twidth ) {
		float off = (r.width()-twidth)/2;
		glColor4f( 1, 1, 1, 1 );
		config->func->drawQuadTexture( fnameTexture, r.x()+off, r.y(), twidth, r.height(), tu, tv, tu1, tv1 );
	}
	else {
		GLfloat nu1 = tu1*r.width()/twidth;
		glColor4f( 1, 1, 1, 1 );
		config->func->drawQuadTexture( fnameTexture, r, tu, tv, nu1, tv1 );
	}
}
